
public abstract class PokerGame extends Thread implements Runnable {
	
	private double bigBlind;
	private double smallBlind;
	private double ante;
	private static PokerStyleFactory factory;
        private static PokerGame pokerGame;
        private static String gameType;
        private static final String CASH = "Cash";  //the two types of poker game
        private static final String TOURNAMENT = "Cash";
	
	protected PokerGame() {  //needs to be singleton
            setSmallBlind(0);
            setBigBlind(0);
            setAnte(0);
	}
        
        public static PokerGame getPokerGame() {
            if(pokerGame == null) {
                if(getGameType().equals(CASH))
                    pokerGame = new CashGame();
                else if(getGameType().equals(TOURNAMENT))
                    pokerGame = new TournamentGame();
            }
            return pokerGame;
	}
        
        public static void setGameType(String type) {
            if(type.equals("-c"))
                gameType = CASH;
            else
                gameType = TOURNAMENT;
        }
        
        public static String getGameType() {
            return gameType;
        }
	
	public double getAnte() {
		return ante;
	}
	
	public void setAnte(double ante) {
		this.ante = ante;
	}
	
	public double getSmallBlind() {
		return smallBlind;
	}
	
	public void setSmallBlind(double sb) {
		smallBlind = sb;
	}
	
	public double getBigBlind() {
		return bigBlind;
	}
	
	public void setBigBlind(double bb) {
		bigBlind = bb;
	}
	
	public PokerStyleFactory getPokerStyleFactory() {
		return factory;
	}
	
	public static void setPokerStyle(String style) {
            if(style.equals("-d"))
                factory = new DrawFactory();
            else
                factory = new HoldEmFactory();
	}
        
	public abstract void initGame();
	
	public abstract void run();
		
	public static void main(String[] args) {
            setGameType(args[0]);
            setPokerStyle(args[1]);
            PokerGame game = getPokerGame();            
	    TableGui gui = TableGui.getTableGui();
            gui.setPokerGame(game);
            gui.setVisible(true);
            game.initGame();
	}
}
